@@ -2,7 +2,6 @@ import copy
 import math
 import os
 import sys
-import random
 import string
 import warnings
 import ipywidgets as widgets
@@ -12,7 +11,7 @@ import scipy as sp
 import sklearn
 from IPython.core.display import display, HTML
 from sklearn.preprocessing import StandardScaler
-from sklearn.utils import check_random_state
+from sklearn.utils import check_random_state, all_estimators
 from sklearn.ensemble import RandomForestClassifier
 from pyexplainer.rulefit import RuleFit
 
@@ -194,11 +193,12 @@ class PyExplainer:
         else:
             print("dep (label column name) should be type 'str'")
             raise TypeError
-        if isinstance(blackbox_model, sklearn.ensemble.RandomForestClassifier):
+        all_clf = all_estimators(type_filter="classifier")
+        supported_algo = [clf[1] for clf in all_clf]
+        if type(blackbox_model) in supported_algo:
             self.blackbox_model = blackbox_model
         else:
-            print("The blackbox_model should be a Random Forest model trained from sklearn ("
-                  "sklearn.ensemble.RandomForestClassifier)")
+            print("The blackbox_model should be a classifier provided by sklearn)")
             raise TypeError
         if isinstance(class_label, list):
             if len(class_label) == 2:
@@ -285,16 +285,16 @@ class PyExplainer:
         """
         # check if X_explain is a DF
         if not isinstance(X_explain, pd.core.frame.DataFrame):
-            print("X_explain (X_test) should be type 'pandas.core.frame.DataFrame'")
-            raise ValueError
+            print("X_explain should be type 'pandas.core.frame.DataFrame'")
+            raise TypeError
         # check if X_explain has the same num of cols as X_train
-        if len(X_explain.columns) != len(X_explain.columns):
+        if len(X_explain.columns) != len(self.X_train.columns):
             print("X_explain should have the same number of columns as X_train")
             raise ValueError
         # check if y_explain is a Series
         if not isinstance(y_explain, pd.core.series.Series):
-            print("y_explain (y_test) should be type 'pandas.core.series.Series'")
-            raise ValueError
+            print("y_explain should be type 'pandas.core.series.Series'")
+            raise TypeError
         self.set_top_k_rules(top_k)
         # Step 1 - Generate synthetic instances
         if search_function.lower() == 'crossoverinterpolation':
@@ -587,7 +587,7 @@ class PyExplainer:
         :obj:`dict`
             A dict with two keys 'synthetic_data' and 'sampled_class_frequency' generated via Crossover and Interpolation.
         """
-        categorical_vars = []
+        # categorical_vars = []
 
         X_train_i = self.X_train.copy()
         # y_train_i = self.y_train.copy()
@@ -635,7 +635,7 @@ class PyExplainer:
             dist = np.linalg.norm(
                 trainset_normalize.sub(np.array(case)), axis=1)
             # Convert distance to a similarity score
-            similarity = np.sqrt(np.exp(-(dist ** 2) / (width ** 2)))
+            similarity = np.exp( -(dist ** 2) / (2 * (width ** 2)) )
             dist_df['dist'] = similarity
             dist_df['t_target'] = target_train
             # get the unique classes of the training set
@@ -694,7 +694,7 @@ class PyExplainer:
             for num in range(0, 1000):
                 rand_rows = train_set_neigh.sample(2)
                 sample_indexes_list = sample_indexes_list + rand_rows.index.values.tolist()
-                similarity_both = dist_df[dist_df.index.isin(rand_rows.index)]
+                # similarity_both = dist_df[dist_df.index.isin(rand_rows.index)]
                 sample_classes = train_class_neigh[train_class_neigh.index.isin(
                     rand_rows.index)]
                 sample_classes = np.array(
@@ -707,6 +707,7 @@ class PyExplainer:
                 new_ins = x + (y - x) * alpha_n
                 new_ins = new_ins.to_frame().T
 
+                """
                 # For Categorical Variables
                 for cat in categorical_vars:
                     x_df = x.to_frame().T
@@ -719,6 +720,7 @@ class PyExplainer:
                         new_ins[cat] = y_df.iloc[0][cat]
                     else:
                         new_ins[cat] = random.choice([x_df.iloc[0][cat], y_df.iloc[0][cat]])
+                """
                 new_ins.name = num
                 new_con_df = new_con_df.append(new_ins, ignore_index=True)
 
@@ -735,12 +737,14 @@ class PyExplainer:
                 z = rand_rows.iloc[2]
                 new_ins = x + (y - z) * mu_f
                 new_ins = new_ins.to_frame().T
+                """
                 # For Categorical Variables get the value of the closest instance to the explained instance
                 for cat in categorical_vars:
                     x_df = x.to_frame().T
                     y_df = y.to_frame().T
                     z_df = z.to_frame().T
                     new_ins[cat] = random.choice([x_df.iloc[0][cat], y_df.iloc[0][cat], z_df.iloc[0][cat]])
+                """
                 new_ins.name = num
                 new_con_df = new_con_df.append(new_ins, ignore_index=True)
 
@@ -798,7 +802,7 @@ class PyExplainer:
         if discretizer is None:
             instance_sample = data_row
             scale = scaler.scale_
-            mean = scaler.mean_
+            # mean = scaler.mean_
             if is_sparse:
                 # Perturb only the non-zero values
                 non_zero_indexes = data_row.nonzero()[1]
@@ -995,7 +999,7 @@ class PyExplainer:
                 slider_widgets.append(slider)
         return slider_widgets
 
-    def on_value_change(self, change):
+    def on_value_change(self, change, debug=False):
         """The callback function for the interactive slider
 
         Whenever the user interacts with the slider,
@@ -1022,7 +1026,10 @@ class PyExplainer:
         bullet_data = self.__get_bullet_data()
         id = int(change['owner'].description.split(" ")[0].strip("#"))
         var_changed = bullet_data[id - 1]['varRef']
-        new_value = change.new
+        if debug:
+            new_value = change['new']
+        else:
+            new_value = change.new
         # modify changed var in X_explain
         X_explain = self.__get_X_explain()
         row_name = self.__get_X_explain().index[0]
